/*
 * This work is part of the Core Imaging Library developed by
 * Visual Analytics and Imaging System Group of the Science Technology
 * Facilities Council, STFC
 *
 * Copyright 2017 Daniil Kazantsev
 * Copyright 2017 Srikanth Nagella, Edoardo Pasca
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "median2d.h"

int floatcomp(const void* elem1, const void* elem2)
{
    if(*(const float*)elem1 < *(const float*)elem2)
        return -1;
    return *(const float*)elem1 > *(const float*)elem2;
}


DLL int
medianfilter_main_float(float* Input, float* Output, int radius, int dimX, int dimY, int dimZ)
{
    int       sizefilter_total;
    int       diameter;
    long      i;
    long      j;
    long      k;
    size_t    index;

    diameter = (2 * radius + 1); /* diameter of the filter's kernel */

    if(dimZ == 0)
    /* 2D filtering */
    {
        sizefilter_total = (int) (powf(diameter, 2));
#pragma omp parallel for shared(Input, Output) private(i, j, index)
        for(j = 0; j < dimY; j++)
        {
            for(i = 0; i < dimX; i++)
            {
                index = (size_t)(j) * dimX + (size_t)(i);
                medfilt2D_float(Input, Output, radius, sizefilter_total, i,
                                j, index, (size_t) dimX, (size_t) dimY);
            }
        }
    }
    else
    /* 3D filtering */
    {
        sizefilter_total = (int) (powf(diameter, 3));
    }
    return 0;
}


int
medfilt2D_float(float* Input, float* Output, int radius, int sizefilter_total,
                long i, long j, size_t index, size_t dimX, size_t dimY)
{
    float*    ValVec;
    long      i_m;
    long      j_m;
    long      i1;
    long      j1;
    long long counter;
    int       midval;
    size_t    index1;
    midval = (sizefilter_total / 2);
    ValVec = (float*) calloc(sizefilter_total, sizeof(float));

    /* filling the allocated vector with the neighbouring values */
    counter = 0LL;
    for(i_m = -radius; i_m <= radius; i_m++)
    {
        i1 = i + i_m;
        if((i1 < 0) || (i1 >= dimX))
            i1 = i;
        for(j_m = -radius; j_m <= radius; j_m++)
        {
            j1 = j + j_m;
            if((j1 < 0) || (j1 >= dimY))
                j1 = j;
            index1 = (size_t)(j1) * dimX + (size_t)(i1);
            ValVec[counter] = Input[index1];
            counter++;
        }
    }
    qsort(ValVec, sizefilter_total, sizeof(float), floatcomp);

    /* perform median filtration */
    Output[index] = ValVec[midval];    

    free(ValVec);
    return 0;
}